#pragma once

#include <iostream>
#include <vector>
#include "rm_road_info.h"

namespace PD
{
    struct PDAsmtVar
    {
        uint32_t pd_obj_id;
        uint8_t pd_ttc_abnml;
        uint8_t pd_td_abnml;
        float pd_latdist;
        float pd_lgtdist;
        uint8_t pd_asmt_reserve01;
        float pd_asmt_reserve02;
        float pd_asmt_reserve03;
        uint8_t pd_asmt_reserve04;
        uint8_t pd_asmt_reserve05;
    };
    struct PDClsnInfo
    {
        uint32_t pd_clsnobj_id;
        float pd_clsnobj_x;
        float pd_clsnobj_y;
        float pd_clsnobj_vx;
        float pd_clsnobj_ttc;
        float pd_clsnobj_td;
        float pd_clsnobj_res01;
        uint8_t pd_clsnobj_res02;
    };
    struct PDObsVar
    {
        dpc::Index pd_passage_id;
        uint8_t pd_ba2pd_actiontype;
        uint8_t pd_obj_att_id;
        bool pd_obj_att_flg;
        std::vector<PDClsnInfo> m_pd_obj_clsninfo;
        uint8_t pd_env_clsn_sts;
        float pd_desireddist;
        float pd_desiredspd;
        uint8_t pd_desiredobj_id;
        float pd_plan_dist;
        float pd_plan_spd;
        float pd_lgtplan_kt;
        float pd_lgtplan_kj;
        std::vector<PDAsmtVar> m_pd_dynmicobj_list;
        // reserved interface
        uint8_t pd_reserve01;
        uint8_t pd_reserve02;
        float pd_reserve03;
        float pd_reserve04;
        uint8_t pd_reserve05;
        uint8_t pd_reserve06;
        uint8_t pd_reserve07;
        float pd_reserve08;
        float pd_reserve09;
        float pd_reserve10;
    };
    struct PDDebug
    {
        std::vector<PDObsVar> m_pd_obs_psg_list;
    };
    struct HVInfo
    {
        float m_distance_alongs;
        float m_velocity;
        uint8_t m_lane_location;
    };
    struct DynamicObjRisk
    {
        std::vector<float> m_obj_long_clsn_ttc;
        std::vector<float> m_obj_long_clsn_td;
        std::vector<float> m_obj_long_ovlp_deg;
        std::vector<float> m_obj_lat_ovlp_deg;
        float m_obj_long_min_distance;
        float m_obj_lat_min_distance;
        float m_obj_long_crs_saftzn_itg;
        float m_obj_lat_crs_saftzn_itg;
        uint32_t m_obj_id;
    };
    struct StnObstcRisk
    {
        // float m_bdrobstc_class;
        float m_bdrlat_min_distance;
        float m_bdrlat_crs_saftzn_itg;
    };
    struct TrffcEffcy
    {
        float m_effcy_longdrv_distance;
        float m_effcy_endspd;
        float m_effcy_avgspd;
    };
    struct Comfort
    {
        float m_cft_lat_acc_max;
        float m_cft_long_acc_max;
        float m_cft_lat_jerk_max;
        float m_cft_long_jerk_max;
    };
    struct LegalPunishment
    {
        bool m_pnshmnt;
    };
    struct RiskEvaluation
    {
        std::vector<DynamicObjRisk> m_dynamicobj_risk_list;
        std::vector<StnObstcRisk> m_stnobstc_risk_list;
        std::vector<TrffcEffcy> m_trffc_effcy_list;
        std::vector<Comfort> m_comfort_list;
        std::vector<LegalPunishment> m_legal_pnshmt_list;
    };
    struct BehaviorNodeLocal
    {
        dpc::Index m_node_id;
        dpc::Index m_parent_node_id;
        RiskEvaluation m_risk_evaluation;
        std::vector<HVInfo> m_hvinfo;
        float m_duration_time;
        bool m_node_clsn_sts;
        // new interface PD2BA
        float desired_spd_limit_val;
        float desired_spd_limt_dist;
        bool spd_limit_flg;
        uint8_t spd_limit_type;

        // #enum:SPD_LIMIT_TYPE
        // uint8 SPD_LIMIT_1ST= 1
        // uint8 SPD_LIMIT_2ND = 2
        // uint8 SPD_LIMTI_CURV = 3
        // uint8 SPD_LIMIT_DEFAULT = 0
    };
    struct PDMdOut
    {
        std::vector<dpc::RoadInfo> pd_roadrodel_vector;
        std::vector<BehaviorNodeLocal> pd_behavior_nodelocal_vector;
    };
    struct PDInputBa
    {
        // BA2PD
        std::vector<dpc::Index> ba_roadmodel_id2pd;
        bool ba_behavior_node_updatedone;
        std::vector<dpc::RoadInfo> ba_roadmodel_vector;
    };
    struct Attention
    {
        uint32_t obj_id;
        bool obj_attention_flg;
        uint8_t obj_attention_grade;
        dpc::Index passage_id;

        // #enum:OBJ_ATTENTION_GRADE
        // uint8 NO_ATTENTION = 0
        // uint8 YELLOW_ATTENTION = 1
        // uint8 ORANGE_ATTENTION = 2
        // uint8 RED_ATTENTION = 3
    };
    struct ObjAttention
    {
        // risk object attention information
        std::vector<Attention> m_obj_attention;
    };
    struct PDVSE
    {
        float m_vse_lgt_vehspd;
        float m_vse_lat_acc;
        float m_vse_lgt_acc;
        float m_vse_yawrate;
        float m_vse_heading_ang;
    };
}
